package net.alviano.menu;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static org.apache.commons.lang3.Validate.*;

public final class Menu {
	public static class Builder {
		private Menu menu = null;
		
		public Builder(MenuDescription description) {
			notNull(description);
			menu = new Menu(new BufferedReader(new InputStreamReader(System.in)), description);
		}
		
		public Builder(BufferedReader reader, MenuDescription description) {
			notNull(reader);
			notNull(description);
			menu = new Menu(reader, description);
		}
		
		public Builder withEntry(MenuEntry entry) {
			notNull(entry);
			menu.entries.add(entry);
			menu.keyToEntry.put(entry.key(), entry);
			return this;
		}
		
		public Menu build() {
			validState(menu != null);
			if(menu.entries.isEmpty()) throw new IllegalStateException("Menu must have at least one entry");
			Menu res = menu;
			menu = null;
			return res;
		}
	}
	
	private final BufferedReader reader;
	private final MenuDescription description;
	private final List<MenuEntry> entries = new ArrayList<MenuEntry>();
	private final Map<MenuKey, MenuEntry> keyToEntry = new HashMap<MenuKey, MenuEntry>();
	
	private Menu(BufferedReader reader, MenuDescription description) {
		this.reader = notNull(reader);
		this.description = notNull(description);
	}
	
	public MenuDescription description() {
		return description;
	}
	
	private void print() {
		System.out.println(String.format("\n\n*** %s ***\n", description));
		for(MenuEntry e : entries) {
			System.out.println(String.format("%s: %s", e.key(), e.description()));
		}
	}
	
	/**
	 * Handle input from user.
	 * @return True if the user want to exit
	 */
	private boolean select() {
		MenuEntry entry = null;
		for(boolean done = false; !done;) {
			try {
				System.out.print("? ");
				String line = readline();
				System.out.println();
				MenuKey key = new MenuKey(line.strip());
				entry = keyToEntry.get(key);
				if(entry != null) {
					entry.onSelected();
					done = true;
				}
			} catch(IOException | IllegalArgumentException e) {
				System.out.println(e.getMessage());
			}
		}
		return entry.isExit(); 
	}
	
	public void run() {
		do{
			print();
		}while(!select());
	}
	
	public String readline() throws IOException {
		return reader.readLine();
	}
}
