package net.alviano.ristorante;

import static org.apache.commons.lang3.Validate.*;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public final class Prezzo implements Comparable<Prezzo> {
	private static final long MIN_CENTS = 0;
	private static final long MAX_CENTS = 100000000000L - 1;
	private static final Pattern parseRegex = Pattern.compile("(\\d{1,9})(?:\\.(\\d{2}))?");
	
	private static Prezzo create(long cents) {
		return new Prezzo((int)(cents / 100), (int)(cents % 100));
	}
	
	public static Prezzo parse(String value) {
		notBlank(value);
		Matcher matcher = parseRegex.matcher(value);
		if(!matcher.matches()) throw new IllegalArgumentException("Prezzo: Errore di parsing");
		int euro = Integer.parseInt(matcher.group(1));
		int cents = matcher.groupCount() == 2 ? Integer.parseInt(matcher.group(2)) : 0;
		return new Prezzo(euro, cents);
	}
	
	private final long value;
	
	public Prezzo(int euro) {
		inclusiveBetween(MIN_CENTS, MAX_CENTS, euro * 100L);
		this.value = euro * 100;
	}
	public Prezzo(int euro, int cents) {
		inclusiveBetween(MIN_CENTS, MAX_CENTS, euro * 100L);
		inclusiveBetween(0, 99, cents);
		this.value = euro * 100L + cents;
		inclusiveBetween(MIN_CENTS, MAX_CENTS, this.value);
	}
	
	public int cents() {
		return (int)(this.value % 100);
	}
	
	public int euro() {
		return (int)(this.value / 100);
	}
	
	public Prezzo add(Prezzo other) {
		notNull(other);
		return Prezzo.create(this.value + other.value);
	}
	
	@Override
	public String toString() {
		return String.format("%d.%02d", euro(), cents());
	}
	
	@Override
	public boolean equals(Object o) {
		if(o == null) return false;
		if(o.getClass() != Prezzo.class) return false;
		Prezzo p = (Prezzo) o;
		return this.value == p.value;
	}
	
	@Override
	public int hashCode() {
		return Long.hashCode(value);
	}

	@Override
	public int compareTo(Prezzo other) {
		return Long.signum(value - other.value);
	}
	
}
